@@ -35,11 +35,7 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 """
-import importlib_metadata
-
 from gaphas.canvas import Canvas
 from gaphas.connector import Handle
 from gaphas.item import Element, Item, Line
-from gaphas.view import GtkView, View
-
-__version__ = importlib_metadata.version("gaphas")
+from gaphas.view import GtkView
