@@ -1,8 +1,7 @@
-"""
-This module contains several flavors of constraint classes.  Each has
-a method `Constraint.solve_for(name)` and a method
-`Constraint.mark_dirty(v)`. These methods are used by the constraint
-solver (`solver.Solver`) to set the variables.
+"""This module contains several flavors of constraint classes.  Each has a
+method `Constraint.solve_for(name)` and a method `Constraint.mark_dirty(v)`.
+These methods are used by the constraint solver (`solver.Solver`) to set the
+variables.
 
 Variables should be of type `solver.Variable`.
 
@@ -27,9 +26,72 @@ New constraint class should derive from Constraint class abstract
 class and implement `Constraint.solve_for(Variable)` method to update
 a variable with appropriate value.
 """
+import logging
 import math
+from typing import Dict, Optional, Tuple
+
+from gaphas.position import Position
+from gaphas.solver import BaseConstraint, Constraint, Variable
+
+log = logging.getLogger(__name__)
+
+
+def constraint(
+    horizontal: Optional[Tuple[Position, Position]] = None,
+    vertical: Optional[Tuple[Position, Position]] = None,
+    left_of: Optional[Tuple[Position, Position]] = None,
+    above: Optional[Tuple[Position, Position]] = None,
+    line: Optional[Tuple[Position, Tuple[Position, Position]]] = None,
+    delta: float = 0.0,
+    align: Optional[float] = None,
+) -> Constraint:
+    """Utility (factory) method to create item's internal constraint between
+    two positions or between a position and a line.
+
+    Position is a tuple of coordinates, i.e. ``(2, 4)``.
+
+    Line is a tuple of positions, i.e. ``((2, 3), (4, 2))``.
+
+    This method shall not be used to create constraints between
+    two different items.
+
+    Created constraint is returned.
+
+    :Parameters:
+        horizontal=(p1, p2)
+        Keep positions ``p1`` and ``p2`` aligned horizontally.
+        vertical=(p1, p2)
+        Keep positions ``p1`` and ``p2`` aligned vertically.
+        left_of=(p1, p2)
+        Keep position ``p1`` on the left side of position ``p2``.
+        above=(p1, p2)
+        Keep position ``p1`` above position ``p2``.
+        line=(p, l)
+        Keep position ``p`` on line ``l``.
+    """
+    cc: Optional[Constraint]
+    if horizontal:
+        p1, p2 = horizontal
+        cc = EqualsConstraint(p1[1], p2[1], delta)
+    elif vertical:
+        p1, p2 = vertical
+        cc = EqualsConstraint(p1[0], p2[0], delta)
+    elif left_of:
+        p1, p2 = left_of
+        cc = LessThanConstraint(p1[0], p2[0], delta)
+    elif above:
+        p1, p2 = above
+        cc = LessThanConstraint(p1[1], p2[1], delta)
+    elif line:
+        pos, line_l = line
+        if align is None:
+            cc = LineConstraint(line=line_l, point=pos)
+        else:
+            cc = LineAlignConstraint(line=line_l, point=pos, align=align, delta=delta)
+    else:
+        raise ValueError("Constraint incorrectly specified")
+    return cc
 
-from gaphas.solver import Projection
 
 # is simple abs(x - y) > EPSILON enough for canvas needs?
 EPSILON = 1e-6
@@ -40,101 +102,8 @@ def _update(variable, value):
         variable.value = value
 
 
-class Constraint:
-    """
-    Constraint base class.
-
-    - _variables - list of all variables
-    - _weakest   - list of weakest variables
-    """
-
-    disabled = False
-
-    def __init__(self, *variables):
-        """
-        Create new constraint, register all variables, and find
-        weakest variables.
-
-        Any value can be added. It is assumed to be a variable if it
-        has a 'strength' attribute.
-        """
-        self._variables = []
-        for v in variables:
-            if hasattr(v, "strength"):
-                self._variables.append(v)
-
-        self.create_weakest_list()
-
-        # Used by the Solver for efficiency
-        self._solver_has_projections = False
-
-    def create_weakest_list(self):
-        """
-        Create list of weakest variables.
-        """
-        # strength = min([v.strength for v in self._variables])
-        strength = min(v.strength for v in self._variables)
-        self._weakest = [v for v in self._variables if v.strength == strength]
-
-    def variables(self):
-        """
-        Return an iterator which iterates over the variables that are
-        held by this constraint.
-        """
-        return self._variables
-
-    def weakest(self):
-        """
-        Return the weakest variable. The weakest variable should be
-        always as first element of Constraint._weakest list.
-        """
-        return self._weakest[0]
-
-    def mark_dirty(self, v):
-        """
-        Mark variable v dirty and if possible move it to the end of
-        Constraint._weakest list to maintain weakest variable
-        invariants (see gaphas.solver module documentation).
-        """
-        weakest = self.weakest()
-        # Fast lane:
-        if v is weakest:
-            self._weakest.remove(v)
-            self._weakest.append(v)
-            return
-
-        # Handle projected variables well:
-        global Projection
-        p = weakest
-        while isinstance(weakest, Projection):
-            weakest = weakest.variable()
-            if v is weakest:
-                self._weakest.remove(p)
-                self._weakest.append(p)
-                return
-
-    def solve(self):
-        """
-        Solve the constraint. This is done by determining the weakest
-        variable and calling solve_for() for that variable. The
-        weakest variable is always in the set of variables with the
-        weakest strength. The least recently changed variable is
-        considered the weakest.
-        """
-        wvar = self.weakest()
-        self.solve_for(wvar)
-
-    def solve_for(self, var):
-        """
-        Solve the constraint for a given variable.
-        The variable itself is updated.
-        """
-        raise NotImplemented
-
-
-class EqualsConstraint(Constraint):
-    """
-    Constraint, which ensures that two arguments ``a`` and ``b`` are equal:
+class EqualsConstraint(BaseConstraint):
+    """Constraint, which ensures that two arguments ``a`` and ``b`` are equal:
 
         a + delta = b
 
@@ -171,11 +140,10 @@ class EqualsConstraint(Constraint):
         )
 
 
-class CenterConstraint(Constraint):
-    """
-    Simple Constraint, takes three arguments: 'a', 'b' and center.
-    When solved, the constraint ensures 'center' is located in the
-    middle of 'a' and 'b'.
+class CenterConstraint(BaseConstraint):
+    """Simple Constraint, takes three arguments: 'a', 'b' and center. When
+    solved, the constraint ensures 'center' is located in the middle of 'a' and
+    'b'.
 
     >>> from gaphas.solver import Variable
     >>> a, b, center = Variable(1.0), Variable(3.0), Variable()
@@ -206,12 +174,10 @@ class CenterConstraint(Constraint):
         _update(self.center, v)
 
 
-class LessThanConstraint(Constraint):
-    """
-    Ensure ``smaller`` is less than ``bigger``. The variable that is
-    passed as to-be-solved is left alone (cause it is the variable
-    that has not been moved lately). Instead the other variable is
-    solved.
+class LessThanConstraint(BaseConstraint):
+    """Ensure ``smaller`` is less than ``bigger``. The variable that is passed
+    as to-be-solved is left alone (cause it is the variable that has not been
+    moved lately). Instead the other variable is solved.
 
     >>> from gaphas.solver import Variable
     >>> a, b = Variable(3.0), Variable(2.0)
@@ -253,9 +219,8 @@ class LessThanConstraint(Constraint):
 ITERLIMIT = 1000  # iteration limit
 
 
-class EquationConstraint(Constraint):
-    """
-    Equation solver using attributes and introspection.
+class EquationConstraint(BaseConstraint):
+    """Equation solver using attributes and introspection.
 
     Takes a function, named arg value (opt.) and returns a Constraint
     object Calling EquationConstraint.solve_for will solve the
@@ -278,7 +243,8 @@ class EquationConstraint(Constraint):
     def __init__(self, f, **args):
         super().__init__(*list(args.values()))
         self._f = f
-        self._args = {}
+        self._args: Dict[str, Optional[Variable]] = {}
+
         # see important note on order of operations in __setattr__ below.
         for arg in f.__code__.co_varnames[0 : f.__code__.co_argcount]:
             self._args[arg] = None
@@ -294,16 +260,12 @@ class EquationConstraint(Constraint):
             return f"EquationConstraint({self._f.__code__.co_name})"
 
     def __getattr__(self, name):
-        """
-        Used to extract function argument values.
-        """
+        """Used to extract function argument values."""
         self._args[name]
         return self.solve_for(name)
 
     def __setattr__(self, name, value):
-        """
-        Sets function argument values.
-        """
+        """Sets function argument values."""
         # Note - once self._args is created, no new attributes can
         # be added to self.__dict__.  This is a good thing as it throws
         # an exception if you try to assign to an arg which is inappropriate
@@ -319,20 +281,17 @@ class EquationConstraint(Constraint):
             object.__setattr__(self, name, value)
 
     def _set(self, **args):
-        """
-        Sets values of function arguments.
-        """
+        """Sets values of function arguments."""
         for arg in args:
             self._args[arg]  # raise exception if arg not in _args
             setattr(self, arg, args[arg])
 
     def solve_for(self, var):
-        """
-        Solve this constraint for the variable named 'arg' in the
-        constraint.
-        """
+        """Solve this constraint for the variable named 'arg' in the
+        constraint."""
         args = {}
         for nm, v in list(self._args.items()):
+            assert v
             args[nm] = v.value
             if v is var:
                 arg = nm
@@ -341,9 +300,7 @@ class EquationConstraint(Constraint):
             var.value = v
 
     def _solve_for(self, arg, args):
-        """
-        Newton's method solver
-        """
+        """Newton's method solver."""
         # args = self._args
         close_runs = 10  # after getting close, do more passes
         if args[arg]:
@@ -356,7 +313,7 @@ class EquationConstraint(Constraint):
             x1 = x0 * 1.1
 
         def f(x):
-            """function to solve"""
+            """function to solve."""
             args[arg] = x
             return self._f(**args)
 
@@ -375,14 +332,14 @@ class EquationConstraint(Constraint):
             else:
                 close_flag = False
             if n > ITERLIMIT:
-                print("Failed to converge; exceeded iteration limit")
+                log.warning("Failed to converge; exceeded iteration limit")
                 break
             slope = (fx1 - fx0) / (x1 - x0)
             if slope == 0:
                 if close_flag:  # we're close but have zero slope, finish
                     break
                 else:
-                    print("Zero slope and not close enough to solution")
+                    log.warning("Zero slope and not close enough to solution")
                     break
             x2 = x0 - fx0 / slope  # New 'x1'
             fx0 = fx1
@@ -392,10 +349,9 @@ class EquationConstraint(Constraint):
         return x1
 
 
-class BalanceConstraint(Constraint):
-    """
-    Ensure that a variable ``v`` is between values specified by
-    ``band`` and in distance proportional from ``band[0]``.
+class BalanceConstraint(BaseConstraint):
+    """Ensure that a variable ``v`` is between values specified by ``band`` and
+    in distance proportional from ``band[0]``.
 
     Consider
 
@@ -442,9 +398,8 @@ class BalanceConstraint(Constraint):
         _update(var, value)
 
 
-class LineConstraint(Constraint):
-    """
-    Ensure a point is kept on a line.
+class LineConstraint(BaseConstraint):
+    """Ensure a point is kept on a line.
 
     Attributes:
      - _line: line defined by tuple ((x1, y1), (x2, y2))
@@ -494,8 +449,7 @@ class LineConstraint(Constraint):
         self._solve()
 
     def _solve(self):
-        """
-        Solve the equation for the connected_handle.
+        """Solve the equation for the connected_handle.
 
         >>> from gaphas.solver import Variable
         >>> line = (Variable(0), Variable(0)), (Variable(30), Variable(20))
@@ -522,9 +476,8 @@ class LineConstraint(Constraint):
         _update(py, y)
 
 
-class PositionConstraint(Constraint):
-    """
-    Ensure that point is always in origin position.
+class PositionConstraint(BaseConstraint):
+    """Ensure that point is always in origin position.
 
     Attributes:
      - _origin: origin position
@@ -538,19 +491,16 @@ class PositionConstraint(Constraint):
         self._point = point
 
     def solve_for(self, var=None):
-        """
-        Ensure that point's coordinates are the same as coordinates of the
-        origin position.
-        """
+        """Ensure that point's coordinates are the same as coordinates of the
+        origin position."""
         x, y = self._origin[0].value, self._origin[1].value
         _update(self._point[0], x)
         _update(self._point[1], y)
 
 
-class LineAlignConstraint(Constraint):
-    """
-    Ensure a point is kept on a line in position specified by align
-    and padding information.
+class LineAlignConstraint(BaseConstraint):
+    """Ensure a point is kept on a line in position specified by align and
+    padding information.
 
     Align is specified as a number between 0 and 1, for example
      0
