@@ -1,22 +1,16 @@
-"""
-Custom decorators.
-"""
+"""Custom decorators."""
+import logging
 import threading
 
-import gi
+from gi.repository import GLib
 
-gi.require_version("Gtk", "3.0")
-from gi.repository import Gtk, GLib
-
-DEBUG_ASYNC = False
+log = logging.getLogger(__name__)
 
 
-class AsyncIO:
-    """
-    Instead of calling the function, schedule an idle handler at a
-    given priority. This requires the async'ed method to be called
-    from within the GTK main loop. Otherwise the method is executed
-    directly.
+class g_async:
+    """Instead of calling the function, schedule an idle handler at a given
+    priority. This requires the async'ed method to be called from within the
+    GTK main loop. Otherwise the method is executed directly.
 
     Note:
         the current implementation of async single mode only works for
@@ -27,14 +21,14 @@ class AsyncIO:
 
     async just works on functions (as long as ``single=False``):
 
-    >>> a = AsyncIO()(lambda: 'Hi')
+    >>> a = g_async()(lambda: 'Hi')
     >>> a()
     'Hi'
 
     Simple method:
 
     >>> class A(object):
-    ...     @AsyncIO(single=False, priority=GLib.PRIORITY_HIGH)
+    ...     @g_async(single=False, priority=GLib.PRIORITY_HIGH)
     ...     def a(self):
     ...         print('idle-a', GLib.main_depth())
 
@@ -42,17 +36,17 @@ class AsyncIO:
     scheduled once).
 
     >>> class B(object):
-    ...     @AsyncIO(single=True)
+    ...     @g_async(single=True)
     ...     def b(self):
     ...         print('idle-b', GLib.main_depth())
 
     Also a timeout property can be provided:
 
     >>> class C(object):
-    ...     @AsyncIO(timeout=50)
+    ...     @g_async(timeout=50)
     ...     def c1(self):
     ...         print('idle-c1', GLib.main_depth())
-    ...     @AsyncIO(single=True, timeout=60)
+    ...     @g_async(single=True, timeout=60)
     ...     def c2(self):
     ...         print('idle-c2', GLib.main_depth())
 
@@ -94,7 +88,12 @@ class AsyncIO:
     it's only executed once.
     """
 
-    def __init__(self, single=False, timeout=0, priority=GLib.PRIORITY_DEFAULT):
+    def __init__(
+        self,
+        single: bool = False,
+        timeout: int = 0,
+        priority: int = GLib.PRIORITY_DEFAULT,
+    ) -> None:
         self.single = single
         self.timeout = timeout
         self.priority = priority
@@ -110,15 +109,13 @@ class AsyncIO:
         async_id = f"_async_id_{func.__name__}"
 
         def wrapper(*args, **kwargs):
-            global getattr, setattr, delattr
             # execute directly if we're not in the main loop
             if GLib.main_depth() == 0:
                 return func(*args, **kwargs)
             elif not self.single:
 
                 def async_wrapper(*aargs):
-                    if DEBUG_ASYNC:
-                        print("async:", func, args, kwargs)
+                    log.debug("async: %s %s %s", func, args, kwargs)
                     func(*args, **kwargs)
 
                 self.source(async_wrapper).attach()
@@ -128,11 +125,10 @@ class AsyncIO:
                 try:
                     if getattr(holder, async_id):
                         return
-                except AttributeError as e:
+                except AttributeError:
 
                     def async_wrapper(*aargs):
-                        if DEBUG_ASYNC:
-                            print("async:", func, args, kwargs)
+                        log.debug("async: %s %s %s", func, args, kwargs)
                         try:
                             func(*args, **kwargs)
                         finally:
@@ -145,8 +141,7 @@ class AsyncIO:
 
 
 def nonrecursive(func):
-    """
-    Enforce a function or method is not executed recursively:
+    """Enforce a function or method is not executed recursively:
 
     >>> class A(object):
     ...     @nonrecursive
@@ -161,9 +156,8 @@ def nonrecursive(func):
     m = threading.Lock()
 
     def wrapper(*args, **kwargs):
-        """
-        Decorate function with a mutex that prohibits recursive execution.
-        """
+        """Decorate function with a mutex that prohibits recursive
+        execution."""
         if m.acquire(False):
             try:
                 return func(*args, **kwargs)
@@ -171,38 +165,3 @@ def nonrecursive(func):
                 m.release()
 
     return wrapper
-
-
-class recursive:
-    """
-    This decorator limits the recursion for a specific function
-
-    >>> class A(object):
-    ...    def __init__(self): self.r = 0
-    ...    @recursive(10)
-    ...    def a(self, x=0):
-    ...        self.r += 1
-    ...        self.a()
-    >>> a = A()
-    >>> a.a()
-    >>> a.r
-    10
-    """
-
-    def __init__(self, limit=10000):
-        self.limit = limit
-
-    def __call__(self, func):
-        def wrapper(*args, **kwargs):
-            try:
-                func._recursion_level += 1
-            except AttributeError:
-                # _recursion_level not present
-                func._recursion_level = 0
-            if func._recursion_level < self.limit:
-                try:
-                    return func(*args, **kwargs)
-                finally:
-                    func._recursion_level -= 1
-
-        return wrapper
