@@ -1,20 +1,23 @@
-"""
-Geometry functions.
+"""Geometry functions.
 
 Rectangle is a utility class for working with rectangles (unions and
 intersections).
 
 A point is represented as a tuple `(x, y)`.
-
 """
+from __future__ import annotations
+
 from math import sqrt
+from typing import Iterable, Optional, Tuple, Union
+
+Point = Tuple[float, float]  # x, y
+Rect = Tuple[float, float, float, float]  # x, y, width, height
 
 
 class Rectangle:
-    """
-    Python Rectangle implementation. Rectangles can be added (union),
-    substituted (intersection) and points and rectangles can be tested
-    to be in the rectangle.
+    """Python Rectangle implementation. Rectangles can be added (union),
+    substituted (intersection) and points and rectangles can be tested to be in
+    the rectangle.
 
     >>> r1= Rectangle(1,1,5,5)
     >>> r2 = Rectangle(3,3,6,7)
@@ -39,7 +42,15 @@ class Rectangle:
     Rectangle(-5, 3, 10, 2)
     """
 
-    def __init__(self, x=0, y=0, width=None, height=None, x1=0, y1=0):
+    def __init__(
+        self,
+        x: float = 0,
+        y: float = 0,
+        width: Optional[float] = None,
+        height: Optional[float] = None,
+        x1: float = 0,
+        y1: float = 0,
+    ):
         if width is None:
             self.x = min(x, x1)
             self.width = abs(x1 - x)
@@ -53,27 +64,27 @@ class Rectangle:
             self.y = y
             self.height = height
 
-    def _set_x1(self, x1):
-        """
-        """
+    @property
+    def x1(self) -> float:
+        return self.x + self.width
+
+    @x1.setter
+    def x1(self, x1: float) -> None:
         width = x1 - self.x
-        if width < 0:
-            width = 0
+        width = max(width, 0)
         self.width = width
 
-    x1 = property(lambda s: s.x + s.width, _set_x1)
+    @property
+    def y1(self) -> float:
+        return self.y + self.height
 
-    def _set_y1(self, y1):
-        """
-        """
+    @y1.setter
+    def y1(self, y1: float) -> None:
         height = y1 - self.y
-        if height < 0:
-            height = 0
+        height = max(height, 0)
         self.height = height
 
-    y1 = property(lambda s: s.y + s.height, _set_y1)
-
-    def expand(self, delta):
+    def expand(self, delta: float) -> None:
         """
         >>> r = Rectangle(-5, 3, 10, 8)
         >>> r.expand(5)
@@ -85,30 +96,34 @@ class Rectangle:
         self.width += delta * 2
         self.height += delta * 2
 
-    def __repr__(self):
+    def tuple(self) -> Tuple[float, float, float, float]:
+        """A type safe version of `tuple(rectangle)`."""
+        return (self.x, self.y, self.width, self.height)
+
+    def __repr__(self) -> str:
         """
         >>> Rectangle(5,7,20,25)
         Rectangle(5, 7, 20, 25)
         """
         if self:
-            return f"({self.__class__.__name__}, {self.x}, {self.y}, {self.width}, {self.height})"
+            return f"{self.__class__.__name__}({self.x}, {self.y}, {self.width}, {self.height})"
         return f"{self.__class__.__name__}()"
 
-    def __iter__(self):
+    def __iter__(self) -> Iterable[float]:
         """
         >>> tuple(Rectangle(1,2,3,4))
         (1, 2, 3, 4)
         """
         return iter((self.x, self.y, self.width, self.height))
 
-    def __getitem__(self, index):
+    def __getitem__(self, index: int) -> float:
         """
         >>> Rectangle(1,2,3,4)[1]
         2
         """
         return (self.x, self.y, self.width, self.height)[index]
 
-    def __bool__(self):
+    def __bool__(self) -> bool:
         """
         >>> r=Rectangle(1,2,3,4)
         >>> if r: 'yes'
@@ -118,19 +133,18 @@ class Rectangle:
         """
         return self.width > 0 and self.height > 0
 
-    def __eq__(self, other):
+    def __eq__(self, other: object) -> bool:
         return (
-            (isinstance(self, type(other)))
+            isinstance(other, Rectangle)
             and self.x == other.x
             and self.y == other.y
             and self.width == other.width
             and self.height == self.height
         )
 
-    def __add__(self, obj):
-        """
-        Create a new Rectangle is the union of the current rectangle
-        with another Rectangle, tuple `(x,y)` or tuple `(x, y, width, height)`.
+    def __add__(self, obj: Union[Rectangle, Rect]) -> Rectangle:
+        """Create a new Rectangle is the union of the current rectangle with
+        another Rectangle, tuple `(x,y)` or tuple `(x, y, width, height)`.
 
         >>> r=Rectangle(5, 7, 20, 25)
         >>> r + (0, 0)
@@ -142,7 +156,7 @@ class Rectangle:
         """
         return Rectangle(self.x, self.y, self.width, self.height).__iadd__(obj)
 
-    def __iadd__(self, obj):
+    def __iadd__(self, obj: Union[Rectangle, Rect]) -> Rectangle:
         """
         >>> r = Rectangle()
         >>> r += Rectangle(5, 7, 20, 25)
@@ -155,7 +169,7 @@ class Rectangle:
         TypeError: Can only add Rectangle or tuple (x, y, width, height), not 'aap'.
         """
         try:
-            x, y, width, height = obj
+            x, y, width, height = obj  # type: ignore[misc]
         except ValueError:
             raise TypeError(
                 f"Can only add Rectangle or tuple (x, y, width, height), not {repr(obj)}."
@@ -171,10 +185,9 @@ class Rectangle:
             self.x, self.y, self.width, self.height = x, y, width, height
         return self
 
-    def __sub__(self, obj):
-        """
-        Create a new Rectangle is the union of the current rectangle
-        with another Rectangle or tuple (x, y, width, height).
+    def __sub__(self, obj: Union[Rectangle, Rect]) -> Rectangle:
+        """Create a new Rectangle is the union of the current rectangle with
+        another Rectangle or tuple (x, y, width, height).
 
         >>> r = Rectangle(5, 7, 20, 25)
         >>> r - (20, 30, 40, 50)
@@ -184,7 +197,7 @@ class Rectangle:
         """
         return Rectangle(self.x, self.y, self.width, self.height).__isub__(obj)
 
-    def __isub__(self, obj):
+    def __isub__(self, obj: Union[Rectangle, Rect]) -> Rectangle:
         """
         >>> r = Rectangle()
         >>> r -= Rectangle(5, 7, 20, 25)
@@ -197,7 +210,7 @@ class Rectangle:
         TypeError: Can only subtract Rectangle or tuple (x, y, width, height), not 'aap'.
         """
         try:
-            x, y, width, height = obj
+            x, y, width, height = obj  # type: ignore[misc]
         except ValueError:
             raise TypeError(
                 f"Can only subtract Rectangle or tuple (x, y, width, height), not {repr(obj)}."
@@ -215,10 +228,9 @@ class Rectangle:
 
         return self
 
-    def __contains__(self, obj):
-        """
-        Check if a point `(x, y)` in inside rectangle `(x, y, width, height)`
-        or if a rectangle instance is inside with the rectangle.
+    def __contains__(self, obj: Union[Rectangle, Rect, Point]) -> bool:
+        """Check if a point `(x, y)` in inside rectangle `(x, y, width,
+        height)` or if a rectangle instance is inside with the rectangle.
 
         >>> r=Rectangle(10, 5, 12, 12)
         >>> (0, 0) in r
@@ -243,13 +255,12 @@ class Rectangle:
         TypeError: Should compare to Rectangle, tuple (x, y, width, height) or point (x, y), not 'aap'.
         """
         try:
-            x, y, width, height = obj
+            x, y, width, height = obj  # type: ignore[misc]
             x1, y1 = x + width, y + width
         except ValueError:
             # point
             try:
-                x, y = obj
-                x1, y1 = obj
+                (x, y) = (x1, y1) = obj  # type: ignore[misc]
             except ValueError:
                 raise TypeError(
                     f"Should compare to Rectangle, tuple (x, y, width, height) or point (x, y), not {repr(obj)}."
@@ -257,11 +268,10 @@ class Rectangle:
         return x >= self.x and x1 <= self.x1 and y >= self.y and y1 <= self.y1
 
 
-def distance_point_point(point1, point2=(0.0, 0.0)):
-    """
-    Return the distance from point ``point1`` to ``point2``.
+def distance_point_point(point1: Point, point2: Point = (0.0, 0.0)) -> float:
+    """Return the distance from point ``point1`` to ``point2``.
 
-    >>> f"{distance_point_point((0,0), (1,1))}:.3f"
+    >>> f"{distance_point_point((0,0), (1,1)):.3f}"
     '1.414'
     """
     dx = point1[0] - point2[0]
@@ -269,11 +279,9 @@ def distance_point_point(point1, point2=(0.0, 0.0)):
     return sqrt(dx * dx + dy * dy)
 
 
-def distance_point_point_fast(point1, point2=(0.0, 0.0)):
-    """
-    Return the distance from point ``point1`` to ``point2``. This
-    version is faster than ``distance_point_point()``, but less
-    precise.
+def distance_point_point_fast(point1: Point, point2: Point = (0.0, 0.0)) -> float:
+    """Return the distance from point ``point1`` to ``point2``. This version is
+    faster than ``distance_point_point()``, but less precise.
 
     >>> distance_point_point_fast((0,0), (1,1))
     2
@@ -283,10 +291,9 @@ def distance_point_point_fast(point1, point2=(0.0, 0.0)):
     return abs(dx) + abs(dy)
 
 
-def distance_rectangle_point(rect, point):
-    """
-    Return the distance (fast) from a rectangle ``(x, y, width,height)``
-    to a ``point``.
+def distance_rectangle_point(rect: Rect, point: Point) -> float:
+    """Return the distance (fast) from a rectangle ``(x, y, width,height)`` to
+    a ``point``.
 
     >>> distance_rectangle_point(Rectangle(0, 0, 10, 10), (11, -1))
     2
@@ -295,9 +302,9 @@ def distance_rectangle_point(rect, point):
     >>> distance_rectangle_point((0, 0, 10, 10), (-1, 11))
     2
     """
-    dx = dy = 0
+    dx = dy = 0.0
     px, py = point
-    rx, ry, rw, rh = tuple(rect)
+    rx, ry, rw, rh = rect
 
     if px < rx:
         dx = rx - px
@@ -312,7 +319,7 @@ def distance_rectangle_point(rect, point):
     return abs(dx) + abs(dy)
 
 
-def point_on_rectangle(rect, point, border=False):
+def point_on_rectangle(rect: Rect, point: Point, border: bool = False) -> Point:
     """
     Return the point on which ``point`` can be projecten on the
     rectangle.  ``border = True`` will make sure the point is bound to
@@ -378,10 +385,11 @@ def point_on_rectangle(rect, point, border=False):
     return px, py
 
 
-def distance_line_point(line_start, line_end, point):
-    """
-    Calculate the distance of a ``point`` from a line. The line is
-    marked by begin and end point ``line_start`` and ``line_end``.
+def distance_line_point(
+    line_start: Point, line_end: Point, point: Point
+) -> Tuple[float, Point]:
+    """Calculate the distance of a ``point`` from a line. The line is marked by
+    begin and end point ``line_start`` and ``line_end``.
 
     A tuple is returned containing the distance and point on the line.
 
@@ -394,7 +402,7 @@ def distance_line_point(line_start, line_end, point):
     >>> d, p = distance_line_point((0., 0.), (2., 4.), point=(2., 2.))
     >>> f"{d:.3f}"
     '0.894'
-    >>> f"({p:.3f}, {p:.3f}"
+    >>> f"({p[0]:.3f}, {p[1]:.3f})"
     '(1.200, 2.400)'
     """
     # The original end point:
@@ -428,11 +436,12 @@ def distance_line_point(line_start, line_end, point):
         )
 
 
-def intersect_line_line(line1_start, line1_end, line2_start, line2_end):
-    """
-    Find the point where the lines (segments) defined by
-    ``(line1_start, line1_end)`` and ``(line2_start, line2_end)``
-    intersect.  If no intersection occurs, ``None`` is returned.
+def intersect_line_line(
+    line1_start: Point, line1_end: Point, line2_start: Point, line2_end: Point
+) -> Optional[Point]:
+    """Find the point where the lines (segments) defined by ``(line1_start,
+    line1_end)`` and ``(line2_start, line2_end)`` intersect.  If no
+    intersection occurs, ``None`` is returned.
 
     >>> intersect_line_line((3, 0), (8, 10), (0, 0), (10, 10))
     (6, 6)
@@ -566,18 +575,15 @@ def intersect_line_line(line1_start, line1_end, line2_start, line2_end):
     return x, y
 
 
-def rectangle_contains(inner, outer):
-    """
-    Returns True if ``inner`` rect is contained in ``outer`` rect.
-    """
+def rectangle_contains(inner: Rect, outer: Rect) -> bool:
+    """Returns True if ``inner`` rect is contained in ``outer`` rect."""
     ix, iy, iw, ih = inner
     ox, oy, ow, oh = outer
     return ox <= ix and oy <= iy and ox + ow >= ix + iw and oy + oh >= iy + ih
 
 
-def rectangle_intersects(recta, rectb):
-    """
-    Return True if ``recta`` and ``rectb`` intersect.
+def rectangle_intersects(recta: Rect, rectb: Rect) -> bool:
+    """Return True if ``recta`` and ``rectb`` intersect.
 
     >>> rectangle_intersects((5,5,20, 20), (10, 10, 1, 1))
     True
@@ -589,10 +595,9 @@ def rectangle_intersects(recta, rectb):
     return ax <= bx + bw and ax + aw >= bx and ay <= by + bh and ay + ah >= by
 
 
-def rectangle_clip(recta, rectb):
-    """
-    Return the clipped rectangle of ``recta`` and ``rectb``. If they
-    do not intersect, ``None`` is returned.
+def rectangle_clip(recta: Rect, rectb: Rect) -> Optional[Rect]:
+    """Return the clipped rectangle of ``recta`` and ``rectb``. If they do not
+    intersect, ``None`` is returned.
 
     >>> rectangle_clip((0, 0, 20, 20), (10, 10, 20, 20))
     (10, 10, 10, 10)
