@@ -1,38 +1,26 @@
-"""
-Module implements guides when moving items and handles around.
-"""
-from functools import reduce
+"""Module implements guides when moving items and handles around."""
+from functools import reduce, singledispatch
 
-from gaphas.aspect import (
-    HandleInMotion,
-    InMotion,
-    ItemHandleInMotion,
-    ItemInMotion,
-    ItemPaintFocused,
-    PaintFocused,
-    singledispatch,
-)
+from gaphas.aspect.handlemove import ElementHandleMove, HandleMove
+from gaphas.aspect.move import ItemMove, Move
+from gaphas.canvas import all_children
 from gaphas.item import Element, Item, Line
+from gaphas.types import Pos
+from gaphas.view import GtkView
 
 
 class ItemGuide:
-    """
-    Get edges on an item, on which we can align the items.
-    """
+    """Get edges on an item, on which we can align the items."""
 
     def __init__(self, item):
         self.item = item
 
     def horizontal(self):
-        """
-        Return horizontal edges (on y axis)
-        """
+        """Return horizontal edges (on y axis)"""
         return ()
 
     def vertical(self):
-        """
-        Return vertical edges (on x axis)
-        """
+        """Return vertical edges (on x axis)"""
         return ()
 
 
@@ -41,24 +29,22 @@ Guide = singledispatch(ItemGuide)
 
 @Guide.register(Element)
 class ElementGuide(ItemGuide):
-    """Guide to align Element items.
-
-    """
+    """Guide to align Element items."""
 
     def horizontal(self):
-        y = self.item.height
-        return (0, y / 2, y)
+        y = self.item.handles()[0].pos.y
+        h = self.item.height
+        return (y, y + h / 2, y + h)
 
     def vertical(self):
-        x = self.item.width
-        return (0, x / 2, x)
+        x = self.item.handles()[0].pos.x
+        w = self.item.width
+        return (x, x + w / 2, x + w)
 
 
 @Guide.register(Line)
 class LineGuide(ItemGuide):
-    """Guide for orthogonal lines.
-
-    """
+    """Guide for orthogonal lines."""
 
     def horizontal(self):
         line = self.item
@@ -98,25 +84,25 @@ class Guides:
 
 
 class GuideMixin:
-    """
-    Helper methods for guides.
-    """
+    """Helper methods for guides."""
 
     MARGIN = 2
 
+    item: Item
+    view: GtkView
+
     def find_vertical_guides(self, item_vedges, pdx, height, excluded_items):
         view = self.view
-        item = self.item
         i2v = self.view.get_matrix_i2v
         margin = self.MARGIN
-        items = []
-        for x in item_vedges:
-            items.append(
-                view.get_items_in_rectangle((x - margin, 0, margin * 2, height))
-            )
+        items = [
+            view.get_items_in_rectangle((x - margin, 0, margin * 2, height))
+            for x in item_vedges
+        ]
+
         try:
             guides = list(
-                map(Guide, reduce(set.union, list(map(set, items))) - excluded_items)
+                map(Guide, reduce(set.union, list(map(set, items))) - excluded_items)  # type: ignore[arg-type]
             )
         except TypeError:
             guides = []
@@ -130,17 +116,16 @@ class GuideMixin:
 
     def find_horizontal_guides(self, item_hedges, pdy, width, excluded_items):
         view = self.view
-        item = self.item
         i2v = self.view.get_matrix_i2v
         margin = self.MARGIN
-        items = []
-        for y in item_hedges:
-            items.append(
-                view.get_items_in_rectangle((0, y - margin, width, margin * 2))
-            )
+        items = [
+            view.get_items_in_rectangle((0, y - margin, width, margin * 2))
+            for y in item_hedges
+        ]
+
         try:
             guides = list(
-                map(Guide, reduce(set.union, list(map(set, items))) - excluded_items)
+                map(Guide, reduce(set.union, list(map(set, items))) - excluded_items)  # type: ignore[arg-type]
             )
         except TypeError:
             guides = []
@@ -155,21 +140,20 @@ class GuideMixin:
         return dy, edges_y
 
     def get_excluded_items(self):
-        """
-        Get a set of items excluded from guide calculation.
-        """
+        """Get a set of items excluded from guide calculation."""
         item = self.item
         view = self.view
+        assert view.model
 
-        excluded_items = set(view.canvas.get_all_children(item))
+        excluded_items = set(all_children(view.model, item))
         excluded_items.add(item)
-        excluded_items.update(view.selected_items)
+        excluded_items.update(view.selection.selected_items)
         return excluded_items
 
     def get_view_dimensions(self):
         try:
             allocation = self.view.get_allocation()
-        except AttributeError as e:
+        except AttributeError:
             return 0, 0
         return allocation.width, allocation.height
 
@@ -183,9 +167,9 @@ class GuideMixin:
         w, h = self.get_view_dimensions()
 
         for x in guides.vertical():
-            view.queue_draw_area(x - 1, 0, x + 2, h)
+            view.queue_redraw()
         for y in guides.horizontal():
-            view.queue_draw_area(0, y - 1, w, y + 2)
+            view.queue_redraw()
 
     def find_closest(self, item_edges, edges):
         delta = 0
@@ -206,12 +190,10 @@ class GuideMixin:
             return 0, ()
 
 
-@InMotion.register(Item)
-class GuidedItemInMotion(GuideMixin, ItemInMotion):
-    """
-    Move the item, lock position on any element that's located at the
-    same location.
-    """
+@Move.register(Element)
+class GuidedItemMove(GuideMixin, ItemMove):
+    """Move the item, lock position on any element that's located at the same
+    location."""
 
     def move(self, pos):
         item = self.item
@@ -245,7 +227,7 @@ class GuidedItemInMotion(GuideMixin, ItemInMotion):
 
         return sink
 
-    def stop_move(self):
+    def stop_move(self, pos):
         self.queue_draw_guides()
         try:
             del self.view.guides
@@ -254,31 +236,31 @@ class GuidedItemInMotion(GuideMixin, ItemInMotion):
             pass
 
 
-@HandleInMotion.register(Item)
-class GuidedItemHandleInMotion(GuideMixin, ItemHandleInMotion):
+@HandleMove.register(Element)
+class GuidedElementHandleMove(GuideMixin, ElementHandleMove):
     """Move a handle and lock the position of other elements.
 
-     Locks the position of another element that's located at the same position.
-
+    Locks the position of another element that's located at the same
+    position.
     """
 
-    def move(self, pos):
+    def move(self, pos: Pos) -> None:
 
         sink = super().move(pos)
 
         if not sink:
             item = self.item
-            handle = self.handle
             view = self.view
+            model = view.model
+            assert model
+
             x, y = pos
             v2i = view.get_matrix_v2i(item)
 
             excluded_items = self.get_excluded_items()
 
             w, h = self.get_view_dimensions()
-
             dx, edges_x = self.find_vertical_guides((x,), 0, h, excluded_items)
-
             dy, edges_y = self.find_horizontal_guides((y,), 0, w, excluded_items)
 
             newpos = x + dx, y + dy
@@ -294,9 +276,10 @@ class GuidedItemHandleInMotion(GuideMixin, ItemHandleInMotion):
 
             self.queue_draw_guides()
 
-            item.request_update()
+            model.request_update(item)
 
-    def stop_move(self):
+    def stop_move(self, pos):
+        super().stop_move(pos)
         self.queue_draw_guides()
         try:
             del self.view.guides
@@ -305,21 +288,23 @@ class GuidedItemHandleInMotion(GuideMixin, ItemHandleInMotion):
             pass
 
 
-@PaintFocused.register(Item)
-class GuidePainter(ItemPaintFocused):
-    def paint(self, context):
+class GuidePainter:
+    def __init__(self, view: GtkView):
+        self.view = view
+
+    def paint(self, _items, cr):
         try:
             guides = self.view.guides
         except AttributeError:
             return
 
-        cr = context.cairo
         view = self.view
         allocation = view.get_allocation()
         w, h = allocation.width, allocation.height
 
         cr.save()
         try:
+            cr.identity_matrix()
             cr.set_line_width(1)
             cr.set_source_rgba(0.0, 0.0, 1.0, 0.6)
             for g in guides.vertical():
