@@ -17,14 +17,31 @@ as a Q-tree. All forms of Quadtrees share some common features:
 
 (From Wikipedia, the free encyclopedia)
 """
+from __future__ import annotations
+
 import operator
+from typing import (
+    Callable,
+    Dict,
+    Generic,
+    Iterable,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    TypeVar,
+)
 
 from gaphas.geometry import rectangle_clip, rectangle_contains, rectangle_intersects
 
+Bounds = Tuple[float, float, float, float]  # x, y, width, height
 
-class Quadtree:
-    """
-    The Quad-tree.
+T = TypeVar("T")
+D = TypeVar("D")
+
+
+class Quadtree(Generic[T, D]):
+    """The Quad-tree.
 
     Rectangles use the same scheme throughout Gaphas: (x, y, width, height).
 
@@ -69,19 +86,10 @@ class Quadtree:
 
     >>> sorted(qtree.find_inside((40, 40, 40, 40)))
     ['13', '14', '15', '16']
-    >>> sorted([qtree.get_bounds(item) for item in qtree.find_inside((40, 40, 40, 40))])
-    [(52, 40, 10, 10), (56, 50, 10, 10), (60, 60, 10, 10), (64, 70, 10, 10)]
-
-    >>> sorted(qtree.find_intersect((40, 40, 20, 20)))
-    ['12', '13', '14', '15']
-    >>> sorted([qtree.get_bounds(item) for item in qtree.find_intersect((40, 40, 20, 20))])
-    [(48, 30, 10, 10), (52, 40, 10, 10), (56, 50, 10, 10), (60, 60, 10, 10)]
-    >>> qtree.rebuild()
     """
 
-    def __init__(self, bounds=(0, 0, 0, 0), capacity=10):
-        """
-        Create a new Quadtree instance.
+    def __init__(self, bounds: Bounds = (0, 0, 0, 0), capacity: int = 10) -> None:
+        """Create a new Quadtree instance.
 
         Bounds is the boundaries of the quadtree. this is fixed and do not
         change depending on the contents.
@@ -89,33 +97,33 @@ class Quadtree:
         Capacity defines the number of elements in one tree bucket (default: 10)
         """
         self._capacity = capacity
-        self._bucket = QuadtreeBucket(bounds, capacity)
+        self._bucket: QuadtreeBucket[T] = QuadtreeBucket(bounds, capacity)
 
-        # Easy lookup item->(bounds, data, clipped bounds) mapping
-        self._ids = {}
+        # Easy lookup item->(bounds, data, clipped bounds)
+        self._ids: Dict[T, Tuple[Bounds, D, Optional[Bounds]]] = {}
 
-    bounds = property(lambda s: s._bucket.bounds)
+    @property
+    def bounds(self) -> Bounds:
+        return self._bucket.bounds
+
+    def resize(self, bounds: Bounds) -> None:
+        """Resize the tree.
 
-    def resize(self, bounds):
-        """
-        Resize the tree.
         The tree structure is rebuild.
         """
         self._bucket = QuadtreeBucket(bounds, self._capacity)
         self.rebuild()
 
-    def get_soft_bounds(self):
-        """
-        Calculate the size of all items in the tree. This size may be
-        beyond the limits of the tree itself.
+    @property
+    def soft_bounds(self) -> Bounds:
+        """Calculate the size of all items in the tree. This size may be beyond
+        the limits of the tree itself.
 
         Returns a tuple (x, y, width, height).
 
         >>> qtree = Quadtree()
         >>> qtree.add('1', (10, 20, 30, 40))
         >>> qtree.add('2', (20, 30, 40, 10))
-        >>> qtree.bounds
-        (0, 0, 0, 0)
         >>> qtree.soft_bounds
         (10, 20, 50, 40)
 
@@ -124,34 +132,22 @@ class Quadtree:
         >>> qtree.bounds
         (0, 0, 0, 0)
         """
-        x_y_w_h = list(
-            zip(
-                *list(
-                    map(
-                        operator.getitem,
-                        iter(list(self._ids.values())),
-                        [0] * len(self._ids),
-                    )
-                )
-            )
-        )
+        x_y_w_h = list(zip(*[d[0] for d in self._ids.values()]))
         if not x_y_w_h:
             return 0, 0, 0, 0
         x0 = min(x_y_w_h[0])
         y0 = min(x_y_w_h[1])
         add = operator.add
-        x1 = max(list(map(add, x_y_w_h[0], x_y_w_h[2])))
-        y1 = max(list(map(add, x_y_w_h[1], x_y_w_h[3])))
-        return (x0, y0, x1 - x0, y1 - y0)
+        x1 = max(map(add, x_y_w_h[0], x_y_w_h[2]))
+        y1 = max(map(add, x_y_w_h[1], x_y_w_h[3]))
+        return x0, y0, x1 - x0, y1 - y0
 
-    soft_bounds = property(get_soft_bounds)
+    def add(self, item: T, bounds: Bounds, data: D) -> None:
+        """Add an item to the tree.
 
-    def add(self, item, bounds, data=None):
-        """
-        Add an item to the tree.
         If an item already exists, its bounds are updated and the item
-        is moved to the right bucket.
-        Data can be used to add some extra info to the item
+        is moved to the right bucket. Data can be used to add some extra
+        info to the item
         """
         # Clip item bounds to fit in top-level bucket
         # Keep original bounds in _ids, for reference
@@ -179,109 +175,89 @@ class Quadtree:
             self._bucket.find_bucket(clipped_bounds).add(item, clipped_bounds)
         self._ids[item] = (bounds, data, clipped_bounds)
 
-    def remove(self, item):
-        """
-        Remove an item from the tree.
-        """
+    def remove(self, item: T) -> None:
+        """Remove an item from the tree."""
         bounds, data, clipped_bounds = self._ids[item]
         del self._ids[item]
         if clipped_bounds:
             self._bucket.find_bucket(clipped_bounds).remove(item)
 
     def clear(self):
-        """
-        Remove all items from the tree.
-        """
+        """Remove all items from the tree."""
         self._bucket.clear()
         self._ids.clear()
 
     def rebuild(self):
-        """
-        Rebuild the tree structure.
-        """
+        """Rebuild the tree structure."""
         # Clean bucket and items:
         self._bucket.clear()
 
-        for item, (bounds, data, _) in list(dict(self._ids).items()):
+        for item, (bounds, data, _) in dict(self._ids).items():
             clipped_bounds = rectangle_clip(bounds, self._bucket.bounds)
             if clipped_bounds:
                 self._bucket.find_bucket(clipped_bounds).add(item, clipped_bounds)
             self._ids[item] = (bounds, data, clipped_bounds)
 
-    def get_bounds(self, item):
-        """
-        Return the bounding box for the given item.
-        """
+    def get_bounds(self, item: T) -> Bounds:
+        """Return the bounding box for the given item."""
         return self._ids[item][0]
 
-    def get_data(self, item):
-        """
-        Return the data for the given item, None if no data was provided.
-        """
+    def get_data(self, item: T) -> D:
+        """Return the data for the given item, None if no data was provided."""
         return self._ids[item][1]
 
-    def get_clipped_bounds(self, item):
-        """
-        Return the bounding box for the given item. The bounding box
-        is clipped on the boundaries of the tree (provided on
-        construction or with resize()).
+    def get_clipped_bounds(self, item: T) -> Optional[Bounds]:
+        """Return the bounding box for the given item.
+
+        The bounding box is clipped on the boundaries of the tree
+        (provided on construction or with resize()).
         """
         return self._ids[item][2]
 
-    def find_inside(self, rect):
-        """
-        Find all items in the given rectangle (x, y, with, height).
+    def find_inside(self, rect: Bounds) -> Set[T]:
+        """Find all items in the given rectangle (x, y, with, height).
+
         Returns a set.
         """
         return set(self._bucket.find(rect, method=rectangle_contains))
 
-    def find_intersect(self, rect):
-        """
-        Find all items that intersect with the given rectangle
-        (x, y, width, height).
+    def find_intersect(self, rect: Bounds) -> Set[T]:
+        """Find all items that intersect with the given rectangle (x, y, width,
+        height).
+
         Returns a set.
         """
         return set(self._bucket.find(rect, method=rectangle_intersects))
 
     def __len__(self):
-        """
-        Return number of items in tree.
-        """
+        """Return number of items in tree."""
         return len(self._ids)
 
-    def __contains__(self, item):
-        """
-        Check if an item is in tree.
-        """
+    def __contains__(self, item: T) -> bool:
+        """Check if an item is in tree."""
         return item in self._ids
 
-    def dump(self):
-        """
-        Print structure to stdout.
-        """
+    def dump(self) -> None:
+        """Print structure to stdout."""
         self._bucket.dump()
 
 
-class QuadtreeBucket:
-    """
-    A node in a Quadtree structure.
-    """
+class QuadtreeBucket(Generic[T]):
+    """A node in a Quadtree structure."""
 
-    def __init__(self, bounds, capacity):
-        """
-        Set bounding box for the node as (x, y, width, height).
-        """
+    def __init__(self, bounds: Bounds, capacity: int):
+        """Set bounding box for the node as (x, y, width, height)."""
         self.bounds = bounds
         self.capacity = capacity
 
-        self.items = {}
-        self._buckets = []
+        self.items: Dict[T, Bounds] = {}
+        self._buckets: List[QuadtreeBucket[T]] = []
 
-    def add(self, item, bounds):
-        """
-        Add an item to the quadtree.
-        The bucket is split when necessary.
-        Items are otherwise added to this bucket, not some sub-bucket.
+    def add(self, item: T, bounds: Bounds) -> None:
+        """Add an item to the quadtree.
+
+        The bucket is split when necessary. Items are otherwise added to
+        this bucket, not some sub-bucket.
         """
         assert rectangle_contains(bounds, self.bounds)
         # create new subnodes if threshold is reached
@@ -304,26 +280,26 @@ class QuadtreeBucket:
         else:
             self.items[item] = bounds
 
-    def remove(self, item):
-        """
-        Remove an item from the quadtree bucket.
-        The item should be contained by *this* bucket (not a sub-bucket).
+    def remove(self, item: T) -> None:
+        """Remove an item from the quadtree bucket.
+
+        The item should be contained by *this* bucket (not a sub-
+        bucket).
         """
         del self.items[item]
 
-    def update(self, item, new_bounds):
-        """
-        Update the position of an item within the current bucket.
-        The item should live in the current bucket, but may be placed in a
-        sub-bucket.
+    def update(self, item: T, new_bounds: Bounds) -> None:
+        """Update the position of an item within the current bucket.
+
+        The item should live in the current bucket, but may be placed in
+        a sub-bucket.
         """
         assert item in self.items
         self.remove(item)
         self.find_bucket(new_bounds).add(item, new_bounds)
 
-    def find_bucket(self, bounds):
-        """
-        Find the bucket that holds a bounding box.
+    def find_bucket(self, bounds: Bounds) -> QuadtreeBucket:
+        """Find the bucket that holds a bounding box.
 
         This method should be used to find a bucket that fits, before
         add() or remove() is called.
@@ -345,10 +321,11 @@ class QuadtreeBucket:
             return self
         return self._buckets[index].find_bucket(bounds)
 
-    def find(self, rect, method):
-        """
-        Find all items in the given rectangle (x, y, with, height).
-        Method can be either the contains or intersects function.
+    def find(
+        self, rect: Bounds, method: Callable[[Bounds, Bounds], bool]
+    ) -> Iterable[T]:
+        """Find all items in the given rectangle (x, y, with, height). Method
+        can be either the contains or intersects function.
 
         Returns an iterator.
         """
@@ -360,9 +337,7 @@ class QuadtreeBucket:
                 yield from bucket.find(rect, method=method)
 
     def clear(self):
-        """
-        Clear the bucket, including sub-buckets.
-        """
+        """Clear the bucket, including sub-buckets."""
         del self._buckets[:]
         self.items.clear()
 
